@@ -22,26 +22,33 @@ import re
 import sys
 import time
 import pexpect
+from signal import Signals
 
 # Default timeout for failing to match.
 TIMEOUT_SECS = 5
 
 UNEXPECTED_SUCCESS = object()
 
+# When rendering fish's output, remove the control sequences that modify terminal state,
+# to avoid confusing the calling terminal. No need to replace things like colors and cursor
+# movement that are harmless and/or will not leak anyway.
+SANITIZE_FOR_PRINTING_RE = re.compile(
+    r"""
+          \x1b\[\?1004[hl]
+        | \x1b\[\?2004[hl]
+        | \x1b\[>4;[10]m
+        | \x1b\[>5u
+        | \x1b\[<1u
+        | \x1b=
+        | \x1b>
+        | \x1b\].*?\x07
+    """,
+    re.VERBOSE)
+
 
 def get_prompt_re(counter):
     """Return a regular expression for matching a with a given prompt counter."""
-    return re.compile(
-        r"""(?:\r\n?|^)   # beginning of line
-            (?:\x1b[\d\[KB(m]*)* # optional colors
-            (?:\[.\]\ )?  # optional vi mode prompt
-         """
-        + (r"prompt\ %d>" % counter)  # prompt with counter
-        + r"""
-            (?:\x1b[\d\[KB(m]*)* # optional colors
-        """,
-        re.VERBOSE,
-    )
+    return re.compile("prompt %d>" % counter)
 
 
 def get_callsite():
@@ -246,15 +253,22 @@ class SpawnedProc(object):
         Report it to stdout, along with the offending call site.
         If 'unmatched' is set, print it to stdout.
         """
+        # Close the process so we can get the status
+        self.spawn.close()
         colors = self.colors()
         failtype = pexpect_error_type(err)
+        # If we get an EOF, we check if the process exited with a signal.
+        # This shows us e.g. if it crashed
+        if failtype == 'EOF' and self.spawn.signalstatus is not None and self.spawn.signalstatus != 0:
+            failtype = "SIGNAL " + Signals(self.spawn.signalstatus).name
+
         fmtkeys = {"failtype": failtype, "pat": escape(pat)}
         fmtkeys.update(**colors)
 
         filename, lineno, code_context = get_callsite()
         fmtkeys["filename"] = filename
         fmtkeys["lineno"] = lineno
-        fmtkeys["code"] = "\n".join(code_context)
+        fmtkeys["code"] = "\n".join([n.strip() for n in code_context if n])
 
         if unmatched:
             print(
@@ -275,9 +289,12 @@ class SpawnedProc(object):
         print("")
         print("{CYAN}When written to the tty, this looks like:{RESET}".format(**colors))
         print("{CYAN}<-------{RESET}".format(**colors))
-        sys.stdout.write(self.spawn.before)
+        sys.stdout.write(SANITIZE_FOR_PRINTING_RE.sub('', self.spawn.before))
         sys.stdout.flush()
-        print("{RESET}\n{CYAN}------->{RESET}".format(**colors))
+        maybe_nl=""
+        if not self.spawn.before.endswith("\n"):
+            maybe_nl="\n{CYAN}(no trailing newline)".format(**colors)
+        print("{RESET}{maybe_nl}{CYAN}------->{RESET}".format(maybe_nl=maybe_nl, **colors))
 
         print("")
 
