@@ -12,11 +12,13 @@ EOF
 
 DOCKER_EXTRA_ARGS=""
 
+export DOCKER_BUILDKIT=1
+
 # Exit on failure.
 set -e
 
 # Get fish source directory.
-FISH_SRC_DIR=$(cd "$( dirname "${BASH_SOURCE[0]}" )"/.. && pwd)
+FISH_SRC_DIR=$(cd "$( dirname "${BASH_SOURCE[0]}" )"/.. >/dev/null && pwd)
 
 # Parse args.
 while [ $# -gt 1 ]; do
@@ -38,7 +40,7 @@ DOCKERFILE=${@:$OPTIND:1}
 test -n "$DOCKERFILE" || usage
 
 # Construct a docker image.
-IMG_TAGNAME="fish_$(basename -s .Dockerfile "$DOCKERFILE")"
+IMG_TAGNAME="ghcr.io/fish-shell/fish-ci/$(basename -s .Dockerfile "$DOCKERFILE"):latest"
 docker build \
     -t "$IMG_TAGNAME" \
     -f "$DOCKERFILE" \
