@@ -1,3 +1,4 @@
+#!/usr/bin/env python3
 # -*- coding: utf-8 -*-
 
 # Run me like this: ./create_manpage_completions.py /usr/share/man/man{1,8}/* > man_completions.fish
@@ -64,7 +65,6 @@ diagnostic_indent = 0
 VERY_VERBOSE, BRIEF_VERBOSE, NOT_VERBOSE = 2, 1, 0
 
 # Pick some reasonable default values for settings
-global VERBOSITY, WRITE_TO_STDOUT, DEROFF_ONLY, KEEP_FILES
 VERBOSITY, WRITE_TO_STDOUT, DEROFF_ONLY, KEEP_FILES = NOT_VERBOSE, False, False, False
 
 
@@ -245,7 +245,7 @@ def remove_groff_formatting(data):
     data = data.replace("\\fB", "")
     data = data.replace("\\fR", "")
     data = data.replace("\\e", "")
-    data = re.sub(".PD( \d+)", "", data)
+    data = re.sub(r".PD( \d+)", "", data)
     data = data.replace(".BI", "")
     data = data.replace(".BR", "")
     data = data.replace("0.5i", "")
@@ -253,14 +253,19 @@ def remove_groff_formatting(data):
     data = data.replace("\\^", "")
     data = data.replace("{ ", "")
     data = data.replace(" }", "")
-    data = data.replace("\ ", "")
-    data = data.replace("\-", "-")
-    data = data.replace("\&", "")
+    data = data.replace(r"\ ", "")
+    data = data.replace(r"\-", "-")
+    data = data.replace(r"\&", "")
     data = data.replace(".B", "")
-    data = data.replace("\-", "-")
+    data = data.replace(r"\-", "-")
     data = data.replace(".I", "")
     data = data.replace("\f", "")
-    data = data.replace("\(cq", "'")
+    data = data.replace(r"\(oq", "'")
+    data = data.replace(r"\(cq", "'")
+    data = data.replace(r"\(aq", "'")
+    data = data.replace(r"\(dq", '"')
+    data = data.replace(r"\(lq", '"')
+    data = data.replace(r"\(rq", '"')
     return data
 
 
@@ -274,17 +279,17 @@ class ManParser(object):
 
 class Type1ManParser(ManParser):
     def is_my_type(self, manpage):
-        return compile_and_search('\.SH "OPTIONS"(.*?)', manpage) != None
+        return compile_and_search(r'\.SH "OPTIONS"(.*?)', manpage) is not None
 
     def parse_man_page(self, manpage):
-        options_section_regex = re.compile('\.SH "OPTIONS"(.*?)(\.SH|\Z)', re.DOTALL)
+        options_section_regex = re.compile(r'\.SH "OPTIONS"(.*?)(\.SH|\Z)', re.DOTALL)
         options_section = re.search(options_section_regex, manpage).group(1)
 
-        options_parts_regex = re.compile("\.PP(.*?)\.RE", re.DOTALL)
+        options_parts_regex = re.compile(r"\.PP(.*?)\.RE", re.DOTALL)
         options_matched = re.search(options_parts_regex, options_section)
         add_diagnostic("Command is %r" % CMDNAME)
 
-        if options_matched == None:
+        if options_matched is None:
             add_diagnostic("Unable to find options")
             if self.fallback(options_section):
                 return True
@@ -292,7 +297,7 @@ class Type1ManParser(ManParser):
                 return True
             return False
 
-        while options_matched != None:
+        while options_matched is not None:
             data = options_matched.group(1)
             last_dotpp_index = data.rfind(".PP")
             if last_dotpp_index != -1:
@@ -320,12 +325,12 @@ class Type1ManParser(ManParser):
 
     def fallback(self, options_section):
         add_diagnostic("Trying fallback")
-        options_parts_regex = re.compile("\.TP( \d+)?(.*?)\.TP", re.DOTALL)
+        options_parts_regex = re.compile(r"\.TP( \d+)?(.*?)\.TP", re.DOTALL)
         options_matched = re.search(options_parts_regex, options_section)
-        if options_matched == None:
+        if options_matched is None:
             add_diagnostic("Still not found")
             return False
-        while options_matched != None:
+        while options_matched is not None:
             data = options_matched.group(2)
             data = remove_groff_formatting(data)
             data = data.strip()
@@ -349,16 +354,16 @@ class Type1ManParser(ManParser):
 
     def fallback2(self, options_section):
         add_diagnostic("Trying last chance fallback")
-        ix_remover_regex = re.compile("\.IX.*")
+        ix_remover_regex = re.compile(r"\.IX.*")
         trailing_num_regex = re.compile("\\d+$")
-        options_parts_regex = re.compile("\.IP (.*?)\.IP", re.DOTALL)
+        options_parts_regex = re.compile(r"\.IP (.*?)\.IP", re.DOTALL)
 
         options_section = re.sub(ix_remover_regex, "", options_section)
         options_matched = re.search(options_parts_regex, options_section)
-        if options_matched == None:
+        if options_matched is None:
             add_diagnostic("Still (still!) not found")
             return False
-        while options_matched != None:
+        while options_matched is not None:
             data = options_matched.group(1)
 
             data = remove_groff_formatting(data)
@@ -386,23 +391,23 @@ class Type1ManParser(ManParser):
 
 class Type2ManParser(ManParser):
     def is_my_type(self, manpage):
-        return compile_and_search("\.SH OPTIONS(.*?)", manpage) != None
+        return compile_and_search(r"\.SH OPTIONS(.*?)", manpage) is not None
 
     def parse_man_page(self, manpage):
-        options_section_regex = re.compile("\.SH OPTIONS(.*?)(\.SH|\Z)", re.DOTALL)
+        options_section_regex = re.compile(r"\.SH OPTIONS(.*?)(\.SH|\Z)", re.DOTALL)
         options_section = re.search(options_section_regex, manpage).group(1)
 
         options_parts_regex = re.compile(
-            "\.[IT]P( \d+(\.\d)?i?)?(.*?)\.([IT]P|UNINDENT|UN|SH)", re.DOTALL
+            r"\.[IT]P( \d+(\.\d)?i?)?(.*?)\.([IT]P|UNINDENT|UN|SH)", re.DOTALL
         )
         options_matched = re.search(options_parts_regex, options_section)
         add_diagnostic("Command is %r" % CMDNAME)
 
-        if options_matched == None:
+        if options_matched is None:
             add_diagnostic("%r: Unable to find options" % self)
             return False
 
-        while options_matched != None:
+        while options_matched is not None:
             data = options_matched.group(3)
             data = remove_groff_formatting(data)
             data = data.strip()
@@ -426,17 +431,17 @@ class Type2ManParser(ManParser):
 
 class Type3ManParser(ManParser):
     def is_my_type(self, manpage):
-        return compile_and_search("\.SH DESCRIPTION(.*?)", manpage) != None
+        return compile_and_search(r"\.SH DESCRIPTION(.*?)", manpage) != None
 
     def parse_man_page(self, manpage):
-        options_section_regex = re.compile("\.SH DESCRIPTION(.*?)(\.SH|\Z)", re.DOTALL)
+        options_section_regex = re.compile(r"\.SH DESCRIPTION(.*?)(\.SH|\Z)", re.DOTALL)
         options_section = re.search(options_section_regex, manpage).group(1)
 
-        options_parts_regex = re.compile("\.TP(.*?)\.TP", re.DOTALL)
+        options_parts_regex = re.compile(r"\.TP(.*?)\.TP", re.DOTALL)
         options_matched = re.search(options_parts_regex, options_section)
         add_diagnostic("Command is %r" % CMDNAME)
 
-        if options_matched == None:
+        if options_matched is None:
             add_diagnostic("Unable to find options section")
             return False
 
@@ -467,19 +472,19 @@ class Type3ManParser(ManParser):
 
 class Type4ManParser(ManParser):
     def is_my_type(self, manpage):
-        return compile_and_search("\.SH FUNCTION LETTERS(.*?)", manpage) != None
+        return compile_and_search(r"\.SH FUNCTION LETTERS(.*?)", manpage) != None
 
     def parse_man_page(self, manpage):
         options_section_regex = re.compile(
-            "\.SH FUNCTION LETTERS(.*?)(\.SH|\Z)", re.DOTALL
+            r"\.SH FUNCTION LETTERS(.*?)(\.SH|\Z)", re.DOTALL
         )
         options_section = re.search(options_section_regex, manpage).group(1)
 
-        options_parts_regex = re.compile("\.TP(.*?)\.TP", re.DOTALL)
+        options_parts_regex = re.compile(r"\.TP(.*?)\.TP", re.DOTALL)
         options_matched = re.search(options_parts_regex, options_section)
         add_diagnostic("Command is %r" % CMDNAME)
 
-        if options_matched == None:
+        if options_matched is None:
             print("Unable to find options section", file=sys.stderr)
             return False
 
@@ -518,9 +523,9 @@ class TypeScdocManParser(ManParser):
         )
 
     def parse_man_page(self, manpage):
-        options_section_regex = re.compile("\.SH OPTIONS(.*?)\.SH", re.DOTALL)
+        options_section_regex = re.compile(r"\.SH OPTIONS(.*?)\.SH", re.DOTALL)
         options_section_matched = re.search(options_section_regex, manpage)
-        if options_section_matched == None:
+        if options_section_matched is None:
             return False
         options_section = options_section_matched.group(1)
 
@@ -528,11 +533,11 @@ class TypeScdocManParser(ManParser):
         options_matched = re.match(options_parts_regex, options_section)
         add_diagnostic("Command is %r" % CMDNAME)
 
-        if options_matched == None:
+        if options_matched is None:
             add_diagnostic("%r: Unable to find options" % self)
             return False
 
-        while options_matched != None:
+        while options_matched is not None:
             # Get first option and move options_section
             option = options_matched.group(1)
             options_section = options_section[options_matched.end() : :]
@@ -568,14 +573,14 @@ class TypeScdocManParser(ManParser):
 
 class TypeDarwinManParser(ManParser):
     def is_my_type(self, manpage):
-        return compile_and_search("\.S[hH] DESCRIPTION", manpage) != None
+        return compile_and_search(r"\.S[hH] DESCRIPTION", manpage) is not None
 
     def trim_groff(self, line):
         # Remove initial period
         if line.startswith("."):
             line = line[1:]
         # Skip leading groff crud
-        while re.match("[A-Z][a-z]\s", line):
+        while re.match(r"[A-Z][a-z]\s", line):
             line = line[3:]
 
         # If the line ends with a space and then a period or comma, then erase the space
@@ -600,7 +605,8 @@ class TypeDarwinManParser(ManParser):
     def groff_replace_escapes(self, line):
         line = line.replace(".Nm", CMDNAME)
         line = line.replace("\\ ", " ")
-        line = line.replace("\& ", "")
+        line = line.replace(r"\& ", "")
+        line = line.replace(".Pp", "")
         return line
 
     def is_option(self, line):
@@ -787,6 +793,26 @@ def parse_manpage_at_path(manpage_path, output_directory):
     if CMDNAME in ignoredcommands:
         return
 
+    # Ignore some commands' gazillion man pages
+    # for subcommands - especially things we already have
+    ignored_prefixes = [
+        "bundle-",
+        "cargo-",
+        "ffmpeg-",
+        "flatpak-",
+        "git-",
+        "npm-",
+        "openssl-",
+        "ostree-",
+        "perf-",
+        "perl",
+        "pip-",
+        "zsh",
+    ]
+    for prefix in ignored_prefixes:
+        if CMDNAME.startswith(prefix):
+            return
+
     # Clear diagnostics
     global diagnostic_indent
     diagnostic_output[:] = []
@@ -825,18 +851,14 @@ def parse_manpage_at_path(manpage_path, output_directory):
 
     manpage = str(manpage)
 
-    # Ignore perl's gazillion man pages
-    ignored_prefixes = ["perl", "zsh"]
-    for prefix in ignored_prefixes:
-        if CMDNAME.startswith(prefix):
-            return
-
     # Ignore the millions of links to BUILTIN(1)
     if "BUILTIN 1" in manpage or "builtin.1" in manpage:
         return
 
     # Clear the output list
     built_command_output[:] = []
+    global already_output_completions
+    already_output_completions = {}
 
     if DEROFF_ONLY:
         parsers = [TypeDeroffManParser()]
@@ -929,9 +951,15 @@ def parse_and_output_man_pages(paths, output_directory, show_progress):
     for manpage_path in paths:
         index += 1
 
-        # Get the "base" command, e.g. gcc.1.gz -> gcc
+        # Get the "base" command, e.g. mkfs.xfs.8.gz -> mkfs.xfs
         man_file_name = os.path.basename(manpage_path)
-        CMDNAME = man_file_name.split(".", 1)[0]
+        # 1. strip the optional compressor suffix
+        CMDNAME, extension = os.path.splitext(man_file_name)
+        # 2. strip mandatory section nu:ber
+        if CMDNAME.endswith((".1", ".2", ".3", ".4", ".5", ".6", ".7", ".8", ".9")):
+            CMDNAME, extension = os.path.splitext(CMDNAME)
+        # 3. XXX strip optional version(?) number?
+        # see comment above `already_output_completions = {}` line
 
         # Show progress if we're doing that
         if show_progress:
@@ -1100,15 +1128,19 @@ if __name__ == "__main__":
             cleanup_autogenerated_completions_in_directory(cleanup_dir)
 
     if not args.file_paths:
-        print("No paths specified")
+        if args.manpath:
+            print("No paths specified and I can't make sense of your MANPATH")
+            print("Please either give paths or set $MANPATH and try again")
+        else:
+            print("No paths specified")
         sys.exit(0)
 
     if not args.stdout and not args.directory:
-        # Default to ~/.local/share/fish/generated_completions/
+        # Default to ~/.cache/fish/generated_completions
         # Create it if it doesn't exist
-        xdg_data_home = os.getenv("XDG_DATA_HOME", "~/.local/share")
+        xdg_cache_home = os.getenv("XDG_CACHE_HOME", "~/.cache")
         args.directory = os.path.expanduser(
-            xdg_data_home + "/fish/generated_completions/"
+            xdg_cache_home + "/fish/generated_completions/"
         )
         try:
             os.makedirs(args.directory)
