@@ -1,8 +1,5 @@
 #!/usr/bin/env python3
 from pexpect_helper import SpawnedProc
-import subprocess
-import sys
-import time
 
 sp = SpawnedProc()
 send, sendline, sleep, expect_prompt, expect_re, expect_str = (
